import json
from typing import List, Union

from zaiclient import config, http
from zaiclient.exceptions.BatchSizeLimitExceededException import BatchSizeLimitExceededException
from zaiclient.exceptions.EmptyBatchException import EmptyBatchException
from zaiclient.request.Request import Request
from zaiclient.request.Events.Event import Event


class EventRequest(Request):

    def __init__(
        self,
        user_id: str,
        item_ids: List[str],
        timestamp: float,
        event_type: str,
        event_values: List[str],
        from_values: List[Union[str, None]],
        is_zai_recommendations: List[bool],
    ) -> None:
        
        events = []
        tmp_timestamp = timestamp
        self._timestamp = timestamp

        super().__init__(http.HTTPMethod.POST, config.COLLECTOR_API_ENDPOINT)
        
        for item_id, event_value, from_value, is_zai_recommendation in zip(item_ids, event_values, from_values, is_zai_recommendations):
            events.append(
                json.loads(Event(
                    user_id=user_id,
                    item_id=item_id,
                    timestamp=tmp_timestamp, 
                    event_type=event_type,
                    event_value=event_value[:500],
                    from_=from_value[:500] if from_value is not None and len(from_value) != 0 else None,
                    is_zai_recommendation=is_zai_recommendation
                ).json(by_alias=True, exclude_none=True))
            )
            tmp_timestamp += config.EPSILON
        
        if len(events) > config.BATCH_REQUEST_CAP:
            raise BatchSizeLimitExceededException()
        
        if len(events) == 0:
            raise EmptyBatchException()
        
        if len(events) == 1:
            self._payload = events[0]
        else:
            self._payload = events

    
    def get_timestamp(self):

        return self._timestamp

    
    def get_path(self, client_id: str) -> str:

        return config.EVENTS_API_PATH


    def get_payload(self, is_test: bool = False):
        
        if is_test:
            if isinstance(self._payload, list):
                for event in self._payload:
                    event["time_to_live"] = config.TEST_EVENT_TIME_TO_LIVE

            else:
                self._payload["time_to_live"] = config.TEST_EVENT_TIME_TO_LIVE

        return self._payload
    

    def get_query_param(self):
        
        return {}


import time
from typing import List, Union
from zaiclient.exceptions.InputTypeNotEqualException import InputTypeNotEqualException
from zaiclient.exceptions.InputLengthNotEqualException import InputLengthNotEqualException
from zaiclient.request.Events.EventRequest import EventRequest


class AddPurchaseEvent(EventRequest):

    __default_event_type = "purchase"

    def __init__(
        self,
        user_id: str,
        item_ids: Union[str, List[str]],
        prices: Union[int, List[int]],
        timestamp: Union[float, None] = None,
        is_zai_rec: Union[bool, List[bool], None] = None
    ):

        if is_zai_rec is None:
            _is_zai_rec = False if isinstance(item_ids, str) else [False] * len(item_ids)
        else:
            _is_zai_rec = is_zai_rec

        if not isinstance(user_id, str):
            raise TypeError("User ID must be a string value.")

        if not (isinstance(item_ids, List) == isinstance(prices, List) and
                isinstance(item_ids, List) == isinstance(_is_zai_rec, List)
        ):
            raise InputTypeNotEqualException()

        if isinstance(item_ids, List) and isinstance(prices, List) and isinstance(_is_zai_rec, List) and not (
            all(isinstance(item_id, str) for item_id in item_ids) and 
            all(isinstance(price, int) for price in prices) and
            all(isinstance(is_zai_rec_, bool) for is_zai_rec_ in _is_zai_rec)
        ):
            raise InputTypeNotEqualException()

        _item_ids = [item_ids] if isinstance(item_ids, str) else item_ids
        _event_values = [str(prices)] if isinstance(prices, int) else [str(price) for price in prices]
        _timestamp = timestamp if timestamp is not None else time.time()
        from_values = [None] * len(_item_ids)
        is_zai_recommendations = [_is_zai_rec] if isinstance(_is_zai_rec, bool) else _is_zai_rec

        if len(_item_ids) != len(_event_values) or len(_item_ids) != len(is_zai_recommendations):
            raise InputLengthNotEqualException()

        super().__init__(
            user_id, _item_ids, _timestamp, self.__default_event_type, _event_values, from_values, is_zai_recommendations
        )



import json
import re
import requests
from typing import List, Union
import warnings

from zaiclient import config
from zaiclient.auth import ZaiHmacAuth
from zaiclient.exceptions.EmptyBatchException import EmptyBatchException
from zaiclient.exceptions.BatchSizeLimitExceededException import BatchSizeLimitExceededException
from zaiclient.exceptions.InputTypeNotEqualException import InputTypeNotEqualException
from zaiclient.exceptions.ZaiClientException import ZaiClientException
from zaiclient.request.Events.EventRequest import EventRequest
from zaiclient.request.Items.ItemRequest import ItemRequest
from zaiclient.request.Recommendations.RecommendationRequest import RecommendationRequest
from zaiclient.request.Request import Request
from zaiclient.response.ItemResponse import ItemResponse
from zaiclient.response.EventLoggerResponse import EventLoggerResponse
from zaiclient.response.RecommendationResponse import RecommendationResponse

class ZaiClient(object):

    def __init__(self, client_id: str, secret: str, connect_timeout: Union[int, float] = config.CONNECT_TIMEOUT_S, read_timeout: Union[int, float] = config.READ_TIMEOUT_S, custom_endpoint: str = ""):

        if type(client_id) != str:
            raise TypeError('Client ID must be a string value.')
        if type(secret) != str:
            raise TypeError('Secret must be a string value.')
        if isinstance(connect_timeout, (int, float)) == False:
            raise TypeError('Connect Timeout must be an integer or a float value.')
        if isinstance(read_timeout, (int, float)) == False:
            raise TypeError('Read Timeout must be an integer or a float value.')
        if type(custom_endpoint) != str:
            raise TypeError('Custom Endpoint must be a string value.')
        if len(custom_endpoint) > 10:
            raise ValueError("Custom Endpoint must be less than or equal to 10.")
        if re.match('^[a-zA-Z0-9-]*$', custom_endpoint) is None:
            raise ValueError("Only alphanumeric characters are allowed for custom endpoint.")
        
        __connect_timeout = connect_timeout
        if connect_timeout <= 0:
            __connect_timeout = config.CONNECT_TIMEOUT_S
        __read_timeout = read_timeout
        if read_timeout <= 0:
            __read_timeout = config.READ_TIMEOUT_S
            
        self.__custom_endpoint = "" if custom_endpoint == "" else f"-{custom_endpoint}"

        self.__client_id = client_id
        self.__auth = ZaiHmacAuth(client_id, secret)
        self.__timeout = (__connect_timeout, __read_timeout)
        self.__session = requests.Session()
        
    
    def __process_base_url(self, base_url: str) -> str:

        return base_url.format(self.__custom_endpoint)


    def __send_request(self, method: str, url: str, payload, headers = {}, params = {}):
        
        response = requests.Response()
        try:
            response = self.__session.request(
                method=method,
                url=url,
                params=params,
                data=None,
                json=payload,
                headers=headers,
                cookies=None,
                files=None,
                auth=self.__auth,
                timeout=self.__timeout,
                verify=True
            )
            response.raise_for_status()
        except requests.HTTPError as http_err:
            raise ZaiClientException(http_err)

        return response.json()


    def send_request(self, request: Union[Request, List[Request]], is_test: bool = False) -> Union[EventLoggerResponse, RecommendationResponse, ItemResponse]:

        response = None
        
        if isinstance(request, EventRequest):
            payload = request.get_payload(is_test=is_test)

            response = self.__send_request(
                method=request.method.value,
                url=self.__process_base_url(request.base_url) + request.get_path(self.__client_id),
                payload=payload,
                params=request.get_query_param()
            )

            return EventLoggerResponse(**response)

        elif isinstance(request, RecommendationRequest):
            payload = request.get_payload(is_test=is_test).dict()

            response = self.__send_request(
                method=request.method.value,
                url=self.__process_base_url(request.base_url) + request.get_path(self.__client_id),
                payload=payload,
                params=request.get_query_param()
            )

            try:
                response['metadata'] = json.loads(response['metadata'])
            except Exception as error:
                warnings.warn(f"Failed to parse the metadata to object, returning an empty object. Error Message: {error}")
                response['metadata'] = {}

            return RecommendationResponse(**response)
        
        elif isinstance(request, ItemRequest):
            payload = request.get_payload(is_test=is_test)

            response = self.__send_request(
                method=request.method.value,
                url=self.__process_base_url(request.base_url) + request.get_path(self.__client_id),
                payload=payload,
                params=request.get_query_param()
            )

            return ItemResponse(**response)
        
        elif isinstance(request, list):
            if len(request) == 0:
                raise EmptyBatchException()
            
            if not isinstance(request[0], Request):
                raise TypeError()

            if not all(isinstance(item_request, type(request[0])) for item_request in request):
                raise InputTypeNotEqualException()
            
            if len(request) > config.BATCH_REQUEST_CAP:
                raise BatchSizeLimitExceededException()

            item_payload = [item.get_payload(is_test=is_test) for item in request]
            _query_params = [item.get_query_param() for item in request]
            
            query_params = {}
            for key in _query_params[0].keys():
                query_params[key] = []

            for query_param in _query_params:
                for key in query_param:
                    if key in query_params:
                        query_params[key].append(query_param[key])

            response = self.__send_request(
                method=request[0].method.value,
                url= self.__process_base_url(request[0].base_url) + request[0].get_path(self.__client_id),
                payload=item_payload,
                params=query_params
            )

            return ItemResponse(**response)
        
        else:
            raise TypeError()


from typing import Union
from pydantic import BaseModel, Field, confloat, conint, constr

class Event(BaseModel):

    user_id: constr(min_length=1, max_length=500)
    item_id: Union[constr(min_length=1, max_length=500), None]
    timestamp: confloat(ge=1_648_871_097., le=2_147_483_647.)
    event_type: constr(min_length=1, max_length=500)
    event_value: constr(min_length=1, max_length=500)
    from_: Union[constr(min_length=1, max_length=500), None] = Field(alias="from")
    is_zai_recommendation: bool
    time_to_live: Union[conint(ge=0), None] = None
    
    class Config:

        allow_population_by_field_name = True

        schema_extra = {
            'example': {
                'user_id': '123456',
                'item_id': 'ABCDEF',
                'timestamp': 1_648_871_097.,
                'event_type': 'product_detail_view',
                'event_value': '1',
                'from': 'home',
                'is_zai_recommendation': True,
                'time_to_live': 60 * 60 * 24
            }
        }


'''Copyright (c) 2022 Z.Ai Inc. ALL RIGHTS RESERVED

Z.Ai official client SDK.
Z.Ai's clients can utilize this SDK to easily connect to the provided API endpoints.
'''

__version__ = '0.1.0'

from zaiclient.client import ZaiClient
from zaiclient.request import *
from zaiclient.response import *
from zaiclient.exceptions import *


from requests import HTTPError

class ZaiClientException(Exception):

    def __init__(self, e: HTTPError):
        self.__status_code = e.response.status_code
        
    def get_http_status_code(self):
        return self.__status_code


from pydantic import constr
from zaiclient import http
from zaiclient.request.Items.ItemRequest import ItemRequest


class AddItem(ItemRequest):

    def __init__(
        self,
        item_id: constr(min_length=1, max_length=500),
        item_name: constr(min_length=1, max_length=2000),
        properties: dict = {}
    ):
        
        super().__init__(http.HTTPMethod.POST, id=item_id, name=item_name, properties=properties)


import time
from typing import List, Union
from zaiclient.exceptions.InputTypeNotEqualException import InputTypeNotEqualException
from zaiclient.exceptions.InputLengthNotEqualException import InputLengthNotEqualException
from zaiclient.request.Events.EventRequest import EventRequest


class AddCustomEvent(EventRequest):

    def __init__(
        self,
        user_id: str,
        item_ids: Union[str, List[str]],
        event_type: str,
        event_values: Union[str, List[str]],
        timestamp: Union[float, None] = None,
        from_: Union[str, List[str], None] = None,
        is_zai_rec: Union[bool, List[bool], None] = None
    ):

        if from_ is None:
            _from = "" if isinstance(item_ids, str) else [""] * len(item_ids)
        else:
            _from = from_

        if is_zai_rec is None:
            _is_zai_rec = False if isinstance(item_ids, str) else [False] * len(item_ids)
        else:
            _is_zai_rec = is_zai_rec

        if not isinstance(user_id, str):
            raise TypeError("User ID must be a string value.")

        if not isinstance(event_type, str):
            raise TypeError("Event Type must be a string value.")

        if (type(item_ids) != type(event_values) or 
            type(item_ids) != type(_from) or 
            isinstance(item_ids, List) != isinstance(_is_zai_rec, List)
        ):
            raise InputTypeNotEqualException()

        if (isinstance(item_ids, List) and isinstance(event_values, List)
            and isinstance(_from, List) and isinstance(_is_zai_rec, List)
            and not (
                all(isinstance(item_id, str) for item_id in item_ids) and
                all(isinstance(event_value, str) for event_value in event_values) and
                all(isinstance(_from_, str) for _from_ in _from) and
                all(isinstance(_is_zai_rec_, bool) for _is_zai_rec_ in _is_zai_rec)
            )
        ):
            raise InputTypeNotEqualException()

        _item_ids = [item_ids] if isinstance(item_ids, str) else item_ids
        _event_values = [event_values] if isinstance(event_values, str) else event_values
        _timestamp = timestamp if timestamp is not None else time.time()
        from_values = [_from] if isinstance(_from, str) else _from
        is_zai_recommendations = [_is_zai_rec] if isinstance(_is_zai_rec, bool) else _is_zai_rec

        if len(_item_ids) != len(_event_values) or len(_item_ids) != len(from_values) or len(_item_ids) != len(is_zai_recommendations):
            raise InputLengthNotEqualException()

        super().__init__(user_id, _item_ids, _timestamp, event_type, _event_values, from_values, is_zai_recommendations)


from datetime import datetime
from typing import Union
from pydantic import BaseModel, confloat, conint, constr


class Item(BaseModel):
    
    item_id: constr(min_length=1, max_length=500)
    item_name: Union[constr(min_length=1, max_length=2000), None] = None
    category_id_1: Union[constr(min_length=0, max_length=2000), None] = None
    category_name_1: Union[constr(min_length=0, max_length=2000), None] = None
    category_id_2: Union[constr(min_length=0, max_length=2000), None] = None
    category_name_2: Union[constr(min_length=0, max_length=2000), None] = None
    category_id_3: Union[constr(min_length=0, max_length=2000), None] = None
    category_name_3: Union[constr(min_length=0, max_length=2000), None] = None
    brand_id: Union[constr(min_length=0, max_length=2000), None] = None
    brand_name: Union[constr(min_length=0, max_length=2000), None] = None
    description: Union[constr(min_length=0, max_length=65535), None] = None
    created_timestamp: Union[datetime, None] = None
    updated_timestamp: Union[datetime, None] = None
    is_active: Union[bool, None] = None
    is_soldout: Union[bool, None] = None
    promote_on: Union[constr(min_length=0, max_length=2000), None] = None
    item_group: Union[constr(min_length=0, max_length=2000), None] = None
    rating: Union[confloat(ge=0.), None] = None
    price: Union[confloat(ge=0.), None] = None
    click_counts: Union[conint(ge=0), None] = None
    purchase_counts: Union[conint(ge=0), None] = None
    image_url: Union[constr(min_length=0, max_length=2000), None] = None
    item_url: Union[constr(min_length=0, max_length=2000), None] = None
    miscellaneous: Union[constr(min_length=0, max_length=65535), None] = None
    


from typing import List
from pydantic import BaseModel, confloat, conint

class RecommendationResponse(BaseModel):

    items: List[str]
    count: conint(ge=0, le=10_000)
    metadata: dict
    timestamp: confloat(ge=1_648_871_097., le=2_147_483_647.)

    class Config:

        schema_extra = {
            'example': {
                'items': ["1234", "1232"],
                'count': 4,
                'metadata': {
                    'recommendation_type': 'homepage'
                },
                'timestamp': 1648835666.825221,
            }
        }


# HTTP Verbs
from enum import Enum


GET = "GET"
POST = "POST"
PUT = "PUT"
PATCH = "PATCH"
HEAD = "HEAD"
DELETE = "DELETE"
OPTIONS = "OPTIONS"

class HTTPMethod(str, Enum):

    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"



from pydantic import BaseModel
from zaiclient.http import HTTPMethod

class Request(object):

    def __init__(
        self,
        method: HTTPMethod,
        base_url: str,
    ):
        
        self.method = method
        self.base_url = base_url


    def get_path(self, client_id: str):

        raise NotImplementedError()
    

    def get_payload(self, is_test: bool):

        raise NotImplementedError()
    

    def get_query_param(self):

        raise NotImplementedError()


from typing import Union
from pydantic import BaseModel, conint, conlist, constr


class Recommendation(BaseModel):

    user_id: Union[constr(min_length=1, max_length=500), None] = None
    item_id: Union[constr(min_length=1, max_length=500), None] = None
    item_ids: Union[conlist(
        item_type=constr(min_length=1, max_length=500),
        min_items=0,
        max_items=10_000
    ), None] = None
    limit: conint(ge=0, le=10_000)
    offset: conint(ge=0, le=10_000) = 0
    options: Union[constr(min_length=0, max_length=1000), None] = None
    recommendation_type: constr(min_length=1, max_length=500)

    class Config:

        schema_extra = {
            'example': {
                'user_id': '123456',
                'limit': 30,
                'offset': 0,
                'recommendation_type': 'homepage'
            }
        }


from zaiclient.exceptions.BatchSizeLimitExceededException import BatchSizeLimitExceededException
from zaiclient.exceptions.EmptyBatchException import EmptyBatchException
from zaiclient.exceptions.InputLengthNotEqualException import InputLengthNotEqualException
from zaiclient.exceptions.InputTypeNotEqualException import InputTypeNotEqualException
from zaiclient.exceptions.ZaiClientException import ZaiClientException


from pydantic import BaseModel, confloat, conlist, conint

from zaiclient.request.Items.Item import Item

class ItemResponse(BaseModel):

    items: conlist(item_type=Item)
    count: conint(ge=0)
    timestamp: confloat(ge=1_648_871_097., le=2_147_483_647.)



from zaiclient.request.Events.AddPageViewEvent import AddPageViewEvent
from zaiclient.request.Events.AddProductDetailViewEvent import AddProductDetailViewEvent
from zaiclient.request.Events.AddSearchEvent import AddSearchEvent
from zaiclient.request.Events.AddLikeEvent import AddLikeEvent
from zaiclient.request.Events.AddCartaddEvent import AddCartaddEvent
from zaiclient.request.Events.AddPurchaseEvent import AddPurchaseEvent
from zaiclient.request.Events.AddRateEvent import AddRateEvent
from zaiclient.request.Events.AddCustomEvent import AddCustomEvent
from zaiclient.request.Events.EventRequest import EventRequest
from zaiclient.request.Events.Event import Event
from zaiclient.request.Recommendations.GetUserRecommendation import GetUserRecommendation
from zaiclient.request.Recommendations.GetRelatedItemsRecommendation import GetRelatedItemsRecommendation
from zaiclient.request.Recommendations.GetRerankingRecommendation import GetRerankingRecommendation
from zaiclient.request.Recommendations.Recommendation import Recommendation
from zaiclient.request.Recommendations.RecommendationRequest import RecommendationRequest
from zaiclient.request.Items.AddItem import AddItem
from zaiclient.request.Items.DeleteItem import DeleteItem
from zaiclient.request.Items.UpdateItem import UpdateItem
from zaiclient.request.Items.Item import Item
from zaiclient.request.Items.ItemRequest import ItemRequest
from zaiclient.request.Request import Request


from zaiclient.response.EventLoggerResponse import EventLoggerResponse
from zaiclient.response.RecommendationResponse import RecommendationResponse
from zaiclient.response.ItemResponse import ItemResponse


from hashlib import sha256
import hmac
import requests
from requests.auth import AuthBase
import time

from zaiclient import config

class ZaiHmacAuth(AuthBase):
    '''Z.Ai client-side authentication'''

    def __init__(self, client_id: str=None, secret: str=None):
        if type(client_id) != str:
            raise TypeError('Client ID must be a string value.')
        if type(secret) != str:
            raise TypeError('Secret must be a string value.')
        
        self.__client_id = client_id
        self.__secret = secret

    def __call__(self, request: requests.Request) -> requests.Request:
        timestamp = str(int(time.time()))
        request.headers[config.ZAI_CLIENT_ID_HEADER] = self.__client_id
        request.headers[config.ZAI_UNIX_TIMESTAMP_HEADER] = timestamp
        request.headers[config.ZAI_AUTHORIZATION_HEADER] = f'ZAi {self.__sign(request, timestamp)}'

        return request

    def __sign(self, request, timestamp):
        path = f'/{"/".join(request.url.split("/")[3:])}'
        query_index = path.find("?")

        if query_index >= 0:
            path = path[:query_index]

        message = f'{path}:{timestamp}'
        signature = hmac.new(
            str.encode(self.__secret, encoding='UTF-8'),
            str.encode(message, encoding='UTF-8'),
            sha256
        ).hexdigest()

        return signature



from pydantic import constr
from zaiclient import http
from zaiclient.request.Items.ItemRequest import ItemRequest


class DeleteItem(ItemRequest):

    def __init__(
        self,
        item_id: constr(min_length=1, max_length=500),
    ):
        
        super().__init__(http.HTTPMethod.DELETE, item_id)

        self.item_id = item_id


    def get_query_param(self):

        return {"id": self.item_id}


# HTTPS
COLLECTOR_API_ENDPOINT = "https://collector-api{}.zaikorea.org"
EVENTS_API_PATH = "/events"
ITEMS_API_PATH = "/items"

ML_API_ENDPOINT = "https://ml-api{}.zaikorea.org"
ML_API_PATH_PREFIX = "/clients/{}/recommenders"
USER_RECOMMENDATION_PATH_PREFIX = "/user-recommendations"
RELATED_ITEMS_RECOMMENDATION_PATH_PREFIX = "/related-items"
RERANKING_RECOMMENDATION_PATH_PREFIX = "/reranking"


HMAC_ALGORITHM = "HmacSHA256"
HMAC_SCHEME = "ZAi"

# HTTP Custom Headers
ZAI_CLIENT_ID_HEADER = "X-ZAI-CLIENT-ID"
ZAI_UNIX_TIMESTAMP_HEADER = "X-ZAI-TIMESTAMP"
ZAI_AUTHORIZATION_HEADER = "X-ZAI-AUTHORIZATION"

BATCH_REQUEST_CAP = 50
EPSILON = 1e-4
CONNECT_TIMEOUT_S = 10
READ_TIMEOUT_S = 30

TEST_EVENT_TIME_TO_LIVE = 60 * 60 * 24 # 1 day


from typing import List, Union
from zaiclient import config, http
from zaiclient.request.Recommendations.Recommendation import Recommendation

from zaiclient.request.Request import Request

class RecommendationRequest(Request):

    def __init__(
        self,
        user_id: Union[str, None],
        item_id: Union[str, None],
        item_ids: Union[List[str], None],
        recommendation_type: str,
        limit: int,
        offset: int = 0,
        options: Union[str, None] = None,
    ):
        
        super().__init__(http.HTTPMethod.POST, config.ML_API_ENDPOINT)

        self._payload = Recommendation(
            user_id=user_id,
            item_id=item_id,
            item_ids=item_ids,
            recommendation_type=recommendation_type,
            limit=limit,
            offset=offset,
            options=options
        )
        
    def get_path(self, client_id: str) -> str:
        
        return ""
    

    def get_payload(self, is_test: bool = False):

        return self._payload
    
    
    def get_query_param(self):
        
        return {}



import json
from typing import Union
from zaiclient import config
from zaiclient.request.Recommendations.RecommendationRequest import RecommendationRequest

class GetUserRecommendation(RecommendationRequest):
    
    __default_offset = 0
    __default_recommendation_type = "homepage"
    
    def __init__(
            self,
            user_id: Union[str, None],
            limit: int,
            offset: int = __default_offset,
            recommendation_type: str = __default_recommendation_type,
            options: Union[dict, None] = None
        ):

        _options = options

        if options is not None:
            _options = json.dumps(options)

        super().__init__(
            user_id=user_id,
            item_id=None,
            item_ids=None,
            limit=limit,
            offset=offset,
            recommendation_type=recommendation_type,
            options=_options
        )

    def get_path(self, client_id: str) -> str:
        
        return config.ML_API_PATH_PREFIX.format(client_id) + config.USER_RECOMMENDATION_PATH_PREFIX


'''Copyright (c) 2022 Z.Ai Inc. ALL RIGHTS RESERVED

Z.Ai official client SDK.
'''

import pathlib
import setuptools

long_description = (pathlib.Path(__file__).parent.joinpath('README.md').read_text())

setuptools.setup(
    name='zaiclient',
    version='4.0.1',
    description='Z.Ai official client SDK.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/zaikorea/zaiclient-python',
    author='Z.Ai Inc.',
    author_email='tech@zaikorea.org',
    license='Proprietary',
    # PyPI package information.
    classifiers=[
        'License :: Other/Proprietary License',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages=setuptools.find_packages(exclude=( "tests",)),
    include_package_data=True,
    install_requires=pathlib.Path('requirements.txt').read_text().splitlines(),
    keywords='zai client sdk api ml',
)



import json
from typing import Union
from zaiclient import config, http
from zaiclient.request.Items.Item import Item
from zaiclient.request.Request import Request

class ItemRequest(Request):

    def __init__(
        self,
        method: http.HTTPMethod,
        id: str,
        name: Union[str, None] = None,
        properties: dict = None
    ):
        
        super().__init__(method=method, base_url=config.COLLECTOR_API_ENDPOINT)
        
        if properties is None:
            properties = {}
            
        properties["item_id"] = id
        properties["item_name"] = name

        self._payload = json.loads(
            Item(**properties)
            .json(exclude_none=True)
        )
    

    def get_payload(self, is_test: bool = False):
        
        return self._payload
    

    def get_path(self, client_id: str):

        return config.ITEMS_API_PATH
    
    
    def get_query_param(self):

        return {}


from pydantic import BaseModel, confloat, constr, conint

class EventLoggerResponse(BaseModel):

    message: constr(min_length=0, max_length=500)
    failure_count: conint(ge=0, le=10000)
    timestamp: confloat(ge=1_648_871_097., le=2_147_483_647.)

    class Config:

        schema_extra = {
            'example': {
                'message': 'The given event was handled successfully.',
                'failed_count': 0,
                'timestamp': 1_648_871_097.
            }
        }


from pydantic import constr
from zaiclient import http
from zaiclient.request.Items.ItemRequest import ItemRequest


class UpdateItem(ItemRequest):

    def __init__(
        self,
        item_id: constr(min_length=1, max_length=500),
        properties: dict = {}
    ):
        
        super().__init__(
            http.HTTPMethod.PUT,
            id=item_id,
            name=properties['item_name'] if 'item_name' in properties else None,
            properties=properties
        )


import time
from typing import Union
from zaiclient.request.Events.EventRequest import EventRequest


class AddRateEvent(EventRequest):

    __default_event_type = "rate"

    def __init__(
        self,
        user_id: str,
        item_id: str,
        rating: Union[float, int],
        timestamp: Union[float, None] = None,
        is_zai_rec: bool = False
    ):

        if not isinstance(user_id, str):
            raise TypeError("User ID must be a string value.")

        if not isinstance(item_id, str):
            raise TypeError("Item ID must be a string value.")

        if not isinstance(rating, ( float, int )):
            raise TypeError("Rating must be a float or integer value.")

        if not isinstance(is_zai_rec, bool):
            raise TypeError("is_zai_rec must be a boolean value.")

        _item_ids = [item_id]
        _event_values = [str(rating)]
        _timestamp = timestamp if timestamp is not None else time.time()
        is_zai_recommendations = [is_zai_rec]

        super().__init__(
            user_id, _item_ids, _timestamp, self.__default_event_type, _event_values, [None], is_zai_recommendations
        )
